@@ -70,6 +70,7 @@ interoptopus::inventory!(
         functions::weird_1,
         functions::visibility,
         patterns::ascii_pointer::pattern_ascii_pointer_1,
+        patterns::ascii_pointer::pattern_ascii_pointer_2,
         patterns::ascii_pointer::pattern_ascii_pointer_len,
         patterns::slice::pattern_ffi_slice_1,
         patterns::slice::pattern_ffi_slice_2,
@@ -87,5 +88,5 @@ interoptopus::inventory!(
         patterns::callbacks::pattern_callback_1,
     ],
     [types::ExtraType<f32>],
-    [patterns::service::SimpleService]
+    [patterns::service::SimpleService, patterns::service::SimpleServiceLifetime]
 );
